from gevent import monkey
monkey.patch_all(socket=True)


import os
from datetime import timedelta, datetime
#from flask_debugtoolbar import DebugToolbarExtension


from flask_cors import CORS
from flask_wtf import CSRFProtect
#from k2.k2rout import *
from flask import Flask, request, session, g
from flask_login import current_user, LoginManager
from flask_socketio import SocketIO, emit
#import eventlet
from ua_parser import user_agent_parser



from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate

from k2.k2logging import configure_logging
from k2.k2trans import K2Babel
from k2.k2cfg import K2


# configure flask app
class K2Flask(Flask):
    def _check_setup_finished(self, f_name: str) -> None:
        pass


# configure flask app
app = K2Flask(__name__)
#app = Flask(__name__)
CORS(app)
app.config['SECRET_KEY'] = K2.secret_key
app.json.sort_keys = K2.json_sort_keys
#app.config.from_object(k2)
app.config['WTF_CSRF_CHECK_DEFAULT'] = False
csrf = CSRFProtect(app)
#eventlet.monkey_patch(socket=True, select=True)
socketio = SocketIO(app, async_mode='gevent', cors_allowed_origins="*") #, async_mode='eventlet', cors_allowed_origins="*", policy_server=False)


# configure PYTHONPATH
pythonpath = os.environ.get('PYTHONPATH', '')
pythonpath += ':.'
os.environ['PYTHONPATH'] = pythonpath
# configure FLASK_APP
os.environ['FLASK_APP'] = 'app.py'


#db
app.config['SQLALCHEMY_DATABASE_URI'] = K2().init_db()
app.config['SQLALCHEMY_POOL_SIZE'] = K2.sglalchemy_pool_size
app.config['SQLALCHEMY_MAX_OVERFLOW'] = K2.sglalchemy_max_overflow
app.config['REMEMBER_COOKIE_DURATION'] = timedelta(days=7)

db = SQLAlchemy()
db.init_app(app)
K2.db = db


#logging
configure_logging()


#migrations
migrate = Migrate(app, db, autogenerate=True)  #directory='../migrations'


#translations
K2.load_babel_translation_directories()
app.config['BABEL_TRANSLATION_DIRECTORIES'] = K2.babel_translation_directories
if K2.current_language == '':
    app.config['BABEL_DEFAULT_LOCALE'] = K2.default_language
else:
    app.config['BABEL_DEFAULT_LOCALE'] = K2.current_language
app.config['BABEL_LANGUAGES'] = K2.languages
app.config['BABEL_DEFAULT_TIMEZONE'] = K2.timezone
#print(app.extensions)
babel = K2Babel(app)
babel.init_app(app, locale_selector=K2.get_locale)


#load components list
K2.component_list()


@app.before_request
def before_request():
    g.start_time = datetime.now()
    session.permanent = True
    app.permanent_session_lifetime = timedelta(minutes=60)
    session.modified = True
    if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                    LoginManager) and  current_user.is_authenticated:
        K2().add_authorized_users(current_user.get_id(),current_user.get_login())


#menu
@app.context_processor
def inject_static_variable():
    K2().search_menu_items()
    K2().search_menu_items_category()
    server_software = request.environ.get('SERVER_SOFTWARE', '')
    context = {
        'menu': K2.menu,
        'menu_category': K2.menu_category,
        'template_name': K2().current_template(),
        'current_user_login': K2().current_user_login,
        'locale' : K2.default_language,
        'request': request,
        'count_update': 0,
        'port': K2.port,
        'proj': K2.proj_config,
        'countreparts': K2().get_user_counterparts_name(),
        'storages': K2().get_user_stoages_name(),
        'project_name': K2().get_user_project_name(),
        'end_time': datetime.now(),
        'domain': K2.domain,
        'domain_comp': '',
        'online_users': K2.authorized_users

    }
    if 'gunicorn' in server_software.lower():
        #context['domain_comp'] = K2.domain + '/components' #для запуску з nginx
        context['domain_comp'] = K2.domain
        
    return context



@app.after_request
def after_request(response):
    # Отримуємо поточний час
    g.end_time = datetime.now()
    if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                    LoginManager) and  current_user.is_authenticated:
        all_routes = K2.menu_url
        if request.path in all_routes:
            try:
                from components.k2site.k2site.views import K2site
                K2site().user_login_save_log(current_user,
                                             user_agent_parser.Parse(request.headers.get('User-Agent')),
                                             request.headers.get('X-Forwarded-For', request.remote_addr),
                                             'Open page',
                                             request.path)
            except:
                pass
    return response


#platform
K2.platform = K2.get_platform()


# flask routing
from routes import *


if __name__ == '__main__':
    # app.config['DEBUG_TB_PROFILER_ENABLED'] = True
    # app.debug = True
    # toolbar = DebugToolbarExtension(app)
    #app.run(use_reloader=False,   port=K2.port)
    socketio.run(app, port=int(K2.port))

    
